// import { useState } from 'preact/hooks'
// import preactLogo from './assets/preact.svg'
// import viteLogo from '/vite.svg'
import './app.sass'
import {ComponentChildren} from "preact";
import {Icon} from "./icons.tsx";
import {Resource, ResourceBundle, resourceTypes} from "./model/resources.ts";
import {useModel} from "./hooks.tsx";
import {NewLairPrice} from "./model/prices.ts";
import {Lair} from "./model/lair.ts";
import * as portraits from "./portraits.ts";
import {ProgressBar, ProgressBarCallback} from "./progressbar.tsx";
import {generateExtensions} from "./model/extensions.ts";
import {TaskType} from "./model/agent.ts";

export function App() {
  return (
    <>
      <Hud />
      <Tiles />
    </>
  )
}

export function Hud() {
  const model = useModel();
  return <div className={"hud"}>
    <div className={"message"}>
      <p>Greetings, <strong>{"Master"}</strong>! You have...</p>
    </div>
    <div className={"resources"}>
      {resourceTypes.map((resource) => {
        let amount = model.resources.get(resource);
        if (amount == 0) return <></>
        return <div class={"resource"}>
          <Icon resource={resource} amount={amount}/> {amount}
        </div>
      })}
    </div>
  </div>
}


export function Tiles() {
  const model = useModel();

  return <div className={"tiles"}>
    {model.lairs.values.map((i) => <LairTile lair={i} />)}
    <ActionsTile />
  </div>
}


export function Tile({children}: { children: ComponentChildren }) {
  return <div class={"tile"}>
    {children}
  </div>
}

export function LairTile({lair}: {lair: Lair}) {
  const model = useModel();
  let extensions = generateExtensions(model);

  let projects = model.lairs.listProjects(extensions, lair.index)
  let fragments = []
  for (let part of projects.parts) {
    if (part.title) {
      fragments.push(<p className={"medium"}><strong>{part.title}</strong></p>);
    }
    let disabled = false;
    if (lair.activeProject) {
      disabled = true;  // can't switch
    }
    fragments.push(<Actions>
      {part.projects.map((p) =>
        <Action text={p.label} cost={{}} callback={() => {
          /*
          p.callback(model);
          model.notify();
           */
          model.startWorkingOn(lair.index, p);
        }} disabled={disabled} />
      )}
    </Actions>)
  }

  return <Tile>
    <div class={"lair"}>
      <p><strong>{lair.name}</strong></p>
      <LairTileAgent agentId={lair.vampire}/>
      <LairTileAgent agentId={lair.thrall}/>
      {fragments}
    </div>
  </Tile>
  // {/* Build rooms */}
  // <Actions>
  //   <Action text={"Build Grounds."} cost={{}} callback={() => alert("party")}></Action>
  // </Actions>
  // {/* Build buildings for room */}
  // <p className={"medium"}><strong>Grounds</strong></p>
  // <Actions>
  //   <Action text={"Build party zone."} cost={{[Resource.Wisdom]: 1}} callback={() => alert("party")}></Action>
  // </Actions>
}

export function LairTileAgent({agentId}: {agentId: number | null}) {
  const model = useModel();

  let name = null;
  let portrait = portraits.blank;
  let progressBarCallback: ProgressBarCallback = () => { return {
      value: null,
      caption: ""
  }}

  if (agentId != null) {
    let agent = model.agents.get(agentId);
    name = agent.name;
    portrait = portraits.suggest(agent.name);
    progressBarCallback = () => {
      let agentActive = model.agents.get(agentId);

      let currentFractionalTurn = model.turn + model.lifecycle.turnProgress();

      let startedAtTurn = agentActive.activeTask.startedAt;
      let willFinishAtTurn = model.turn + agentActive.activeTask.remainingTurns;

      let value = (currentFractionalTurn - startedAtTurn) / (willFinishAtTurn - startedAtTurn);
      // TODO: Get progress for agent
      return {
        value: agentActive.activeTask.taskType == TaskType.Idle ? null : value,
        caption: agentActive.activeTask.caption,
      }
    }
  }

  let displayName = name ?? "-"
  let portraitAltText = `portrait (${name ?? 'blank'})`

  return <div class={"agent"}>
    <img class={"portrait"} src={portrait} alt={portraitAltText} />
    <div class={"caption"}>
      <div class={"medium"}>{displayName}</div>
      <ProgressBar callback={progressBarCallback}/>
    </div>
  </div>
}

export function ActionsTile({}) {
  const model = useModel();

  return <Tile>
    <Actions>
      <Action text={"Build new lair."} cost={NewLairPrice} callback={
        () => model.buildNewLair()
      }/>
    </Actions>
  </Tile>
}

export function Actions({children}: {children: ComponentChildren}) {
  return <div class={"actions"}>
    {children}
  </div>

}

export function Action({text, cost, callback, disabled}: {text: string, cost: ResourceBundle, callback: () => void, disabled?: boolean}) {
  const model = useModel();

  let available = true;
  if (disabled) {
    available = false
  }
  if (callback == null) {
    available = false;
  }
  if (!model.resources.canSpend(cost)) {
    available = false;
  }

  return <div
    className={"action " + (available ? "available " : "unavailable ")}
    onClick={available ? callback : undefined}
  >
    <div className={"text"}>
      <p>{text}</p>
    </div>
    <div className={"cost"}>
      <ResourceCostFragment cost={cost} />
    </div>
  </div>
}

export function ResourceCostFragment({cost}: {cost: ResourceBundle}) {
  return <>{resourceTypes.map((resource: Resource) => {
      let amount = cost[resource];
      if (amount == 0 || amount == undefined) return <></>
      return <div class={"resource"}>
        <Icon resource={resource} amount={amount}/> {amount}
      </div>
    })}</>;
}