import {generateExtensions, LairExtensions} from "./extensions.ts";
import {Model} from "./model.ts";

export type Lair = {
  index: number,
  name: string,
  vampire: number | null
  thrall: number | null

  extensions: Array<string>,
  objects: Array<string>,
  progress: Record<string, number>,
  activeProject: ActiveProject | null,
}

export type ActiveProject = {
  id: string
  verb: string
  remainingTurns: number
}

export class Lairs {
  values: Array<Lair> = []

  generate(): number {
    let i = this.values.length;
    this.values.push(generateLair(i));
    return i
  }

  setVampire(lair: number, vampire: number | null) {
    this.values[lair].vampire = vampire;
  }

  startWorkingOn(lair: number, project: LairProject) {
    this.values[lair].activeProject = {
      id: project.id,
      verb: project.verb,
      remainingTurns: project.turns,
    }
  }

  updateProjects(model: Model) {
    for (let lair of this.values) {
      let project = lair.activeProject;
      if (project == null) {
        continue;
      }
      if (project.remainingTurns > 0) {
        project.remainingTurns -= 1;
      }

      let extensions = generateExtensions(model);
      if (project.remainingTurns == 0) {
        let projects = this.listProjects(extensions, lair.index)
        let realProject = projects.getById(project.id);
        realProject?.callback(model);
        lair.activeProject = null;
      }
    }

  }

  listProjects(extensions: LairExtensions, lairIx: number): LairProjects {
    let projects = new LairProjects();
    let lair = this.values[lairIx];

    // first: what extensions can be built?
    let unbuilt = []
    for (let extension of extensions.extensions) {
      if (lair.extensions.indexOf(extension.id) >= 0) {
        // already built
        continue;
      }
      unbuilt.push(extension)
    }

    if (unbuilt.length > 0) {
      let section = new LairProjectsSection(null);
      unbuilt.forEach((u) => {
        section.add(new LairProject(
          `addExtension:${u.id}`,
          u.projectName,
          u.verb,
          u.turns,
          (model) => {
          model.lairs.values[lairIx].extensions.push(u.id)
          if (u.payoff != null) {
            u.payoff(model);
          }
        }))
      })
      projects.add(section)
    }

    // next: in each extension, which objects can we build or interact with?
    for (let extensionId of lair.extensions) {
      // find the base implementation
      let extensionData = null;
      for (let realExtension of extensions.extensions) {
        if (realExtension.id == extensionId) {
          extensionData = realExtension;
          break;
        }
      }
      if (extensionData == null) { continue; }

      // now figure out what objects we have (if any)
      let section = new LairProjectsSection(extensionData.extensionName);
      (extensions.extensionObjects[extensionId] ?? []).forEach((o) => {
        let wasBuilt = lair.objects.indexOf(o.id) >= 0;

        if (!wasBuilt) {
          section.add(new LairProject(
            `build:${o.id}`,
            o.name,
            o.verb,
            o.turns,
            (model) => {
              model.lairs.values[lairIx].objects.push(o.id)
              if (o.payoff != null) {
                o.payoff(model);
              }
            }
          ))
        } else {
          (extensions.objectInteractions[o.id] ?? []).forEach((i) => {
            section.add(new LairProject(
              `interact:${o.id}`,
              i.name,
              i.verb,
              i.turns,
              (model) => {
                if (i.payoff != null) {
                  i.payoff(model);
                }
              }
            ))
          })
        }
      });
      projects.add(section);
    }

    return projects
  }
}

function generateLair(index: number) {
  return {
    index: index,
    name: "Pyrex's Castle",
    vampire: null,
    thrall: null,

    extensions: [],
    objects: [],
    progress: {},
    activeProject: null,
  }
}

// == this is the menu, which is fairly closely coupled to React ==
export class LairProjects {
  parts: Array<LairProjectsSection> = [];

  add(section: LairProjectsSection) {
    if (section.projects.length == 0) {
      return;
    }
    this.parts.push(section)
  }

  getById(id: string): LairProject | null {
    for (let part of this.parts) {
      for (let project of part.projects) {
        if (project.id == id) { return project; }
      }
    }
    return null;
  }
}

export class LairProjectsSection {
  title: string | null
  projects: Array<LairProject> = []

  constructor(title: string | null) {
    this.title = title
  }

  add(project: LairProject) {
    this.projects.push(project)
  }
}

export class LairProject {
  id: string
  label: string
  verb: string
  turns: number
  callback: (model: Model) => void

  constructor(id: string, label: string, verb: string, turns: number, callback: (model: Model) => void) {
    this.id = id
    this.label = label
    this.verb = verb
    this.turns = turns
    this.callback = callback
  }
}